@@ -4,7 +4,8 @@ Takes an annotation from a dataset and turns it into the
 ground truth for a field.
 """
 
-from .annrescaler import AnnRescaler
-from .factory import cli, configure, factory, factory_head
+from .annrescaler import AnnRescaler, AnnRescalerDet
+from .factory import cli, configure
 from .caf import Caf
 from .cif import Cif
+from .cifdet import CifDet
