@@ -8,6 +8,7 @@ https://github.com/daquexian/onnx-simplifier
 """
 
 import argparse
+import logging
 import shutil
 
 import torch
@@ -25,63 +26,47 @@ try:
 except ImportError:
     onnxsim = None
 
-
-class GetPif(torch.nn.Module):
-    def forward(self, heads):  # pylint: disable=arguments-differ
-        return heads[0]
+LOG = logging.getLogger(__name__)
 
 
-class GetPifC(torch.nn.Module):
-    def forward(self, heads):  # pylint: disable=arguments-differ
-        return heads[0][0]
+def image_size_warning(basenet_stride, input_w, input_h):
+    if input_w % basenet_stride != 1:
+        LOG.warning(
+            'input width (%d) should be a multiple of basenet '
+            'stride (%d) + 1: closest are %d and %d',
+            input_w, basenet_stride,
+            (input_w - 1) // basenet_stride * basenet_stride + 1,
+            ((input_w - 1) // basenet_stride + 1) * basenet_stride + 1,
+        )
+
+    if input_h % basenet_stride != 1:
+        LOG.warning(
+            'input height (%d) should be a multiple of basenet '
+            'stride (%d) + 1: closest are %d and %d',
+            input_h, basenet_stride,
+            (input_h - 1) // basenet_stride * basenet_stride + 1,
+            ((input_h - 1) // basenet_stride + 1) * basenet_stride + 1,
+        )
 
 
-def apply(model, outfile, verbose=True):
-    # dummy_input = torch.randn(1, 3, 193, 257)
-    dummy_input = torch.randn(1, 3, 97, 129)
-    # model = torch.nn.Sequential(model, GetPifC())
+def apply(model, outfile, verbose=True, input_w=129, input_h=97):
+    image_size_warning(model.base_net.stride, input_w, input_h)
 
-    # Providing input and output names sets the display names for values
-    # within the model's graph. Setting these does not change the semantics
-    # of the graph; it is only for readability.
-    #
-    # The inputs to the network consist of the flat list of inputs (i.e.
-    # the values you would pass to the forward() method) followed by the
-    # flat list of parameters. You can partially specify names, i.e. provide
-    # a list here shorter than the number of inputs to the model, and we will
-    # only set that subset of names, starting from the beginning.
-    input_names = ['input_batch']
-    # output_names = [
-    #     'pif_c',
-    #     'pif_r',
-    #     'pif_b',
-    #     'pif_s',
-    #     'paf_c',
-    #     'paf_r1',
-    #     'paf_r2',
-    #     'paf_b1',
-    #     'paf_b2',
-    # ]
-    output_names = ['cif', 'caf']
+    # configure
+    openpifpaf.network.heads.CompositeField3.inplace_ops = False
+
+    dummy_input = torch.randn(1, 3, input_h, input_w)
 
     torch.onnx.export(
         model, dummy_input, outfile, verbose=verbose,
-        input_names=input_names, output_names=output_names,
-        # opset_version=10,
-        # do_constant_folding=True,
-        # dynamic_axes={  # TODO: gives warnings
-        #     'input_batch': {0: 'batch', 2: 'height', 3: 'width'},
-        #     'pif_c': {0: 'batch', 2: 'fheight', 3: 'fwidth'},
-        #     'pif_r': {0: 'batch', 3: 'fheight', 4: 'fwidth'},
-        #     'pif_b': {0: 'batch', 2: 'fheight', 3: 'fwidth'},
-        #     'pif_s': {0: 'batch', 2: 'fheight', 3: 'fwidth'},
-
-        #     'paf_c': {0: 'batch', 2: 'fheight', 3: 'fwidth'},
-        #     'paf_r1': {0: 'batch', 3: 'fheight', 4: 'fwidth'},
-        #     'paf_b1': {0: 'batch', 2: 'fheight', 3: 'fwidth'},
-        #     'paf_r2': {0: 'batch', 3: 'fheight', 4: 'fwidth'},
-        #     'paf_b2': {0: 'batch', 2: 'fheight', 3: 'fwidth'},
-        # },
+        input_names=['input_batch'], output_names=['cif', 'caf'],
+        # keep_initializers_as_inputs=True,
+        opset_version=11,
+        do_constant_folding=True,
+        dynamic_axes={
+            "input": {0: 'dynamic'},
+            "cif": {0: 'dynamic'},
+            "caf": {0: 'dynamic'}}
     )
 
 
@@ -121,7 +106,8 @@ def simplify(infile, outfile=None):
         infile = infile.replace('.onnx', '.unsimplified.onnx')
         shutil.copyfile(outfile, infile)
 
-    simplified_model = onnxsim.simplify(infile, check_n=0, perform_optimization=False)
+    simplified_model, check_ok = onnxsim.simplify(infile, check_n=3, perform_optimization=False)
+    assert check_ok
     onnx.save(simplified_model, outfile)
 
 
@@ -139,17 +125,23 @@ def main():
     parser.add_argument('--version', action='version',
                         version='OpenPifPaf {version}'.format(version=openpifpaf.__version__))
 
-    parser.add_argument('--checkpoint', default='resnet50')
+    openpifpaf.network.Factory.cli(parser)
+
     parser.add_argument('--outfile', default='openpifpaf-resnet50.onnx')
     parser.add_argument('--simplify', dest='simplify', default=False, action='store_true')
     parser.add_argument('--polish', dest='polish', default=False, action='store_true',
                         help='runs checker, optimizer and shape inference')
     parser.add_argument('--optimize', dest='optimize', default=False, action='store_true')
     parser.add_argument('--check', dest='check', default=False, action='store_true')
+    parser.add_argument('--input-width', type=int, default=129)
+    parser.add_argument('--input-height', type=int, default=97)
     args = parser.parse_args()
 
-    model, _ = openpifpaf.network.factory(checkpoint=args.checkpoint)
-    apply(model, args.outfile)
+    openpifpaf.network.Factory.configure(args)
+
+    model, _ = openpifpaf.network.Factory().factory()
+
+    apply(model, args.outfile, input_w=args.input_width, input_h=args.input_height)
     if args.simplify:
         simplify(args.outfile)
     if args.optimize:
