@@ -4,12 +4,14 @@ import random
 import pysparkling
 import torch
 
-from .decoder.instance_scorer import InstanceScorer
+from .decoder.utils.instance_scorer import InstanceScorer
 from . import show
 
 DATA_FILE = ('outputs/resnet101block5-pif-paf-edge401-190412-151013.pkl'
              '.decodertraindata-edge641-samples0.json')
 
+# pylint: skip-file
+
 
 def plot_training_data(train_data, val_data, entry=0, entryname=None):
     train_x, train_y = train_data
@@ -81,6 +83,7 @@ def train_epoch(model, loader, optimizer):
 
     return epoch_loss / len(loader)
 
+
 def val_epoch(model, loader):
     epoch_loss = 0.0
     with torch.no_grad():
