@@ -5,15 +5,17 @@ import torchvision
 from .annotations import AnnotationJitter, NormalizeAnnotations
 from .compose import Compose
 from .crop import Crop
+from .encoders import Encoders
 from .hflip import HFlip
 from .image import Blur, ImageTransform, JpegCompression
 from .minsize import MinSize
 from .multi_scale import MultiScale
 from .pad import CenterPad, CenterPadTight, SquarePad
 from .preprocess import Preprocess
-from .random import DeterministicEqualChoice, RandomApply
-from .rotate import RotateBy90
+from .random import DeterministicEqualChoice, RandomApply, RandomChoice
+from .rotate import RotateBy90, RotateUniform
 from .scale import RescaleAbsolute, RescaleRelative, ScaleMix
+from .toannotations import ToAnnotations, ToCrowdAnnotations, ToDetAnnotations, ToKpAnnotations
 from .unclipped import UnclippedArea, UnclippedSides
 
 
@@ -30,9 +32,28 @@ EVAL_TRANSFORM = Compose([
 TRAIN_TRANSFORM = Compose([
     NormalizeAnnotations(),
     ImageTransform(torchvision.transforms.ColorJitter(
-        brightness=0.1, contrast=0.1, saturation=0.1, hue=0.1)),
+        brightness=0.4, contrast=0.1, saturation=0.4, hue=0.1)),
     RandomApply(JpegCompression(), 0.1),  # maybe irrelevant for COCO, but good for others
     # RandomApply(Blur(), 0.01),  # maybe irrelevant for COCO, but good for others
     ImageTransform(torchvision.transforms.RandomGrayscale(p=0.01)),
     EVAL_TRANSFORM,
 ])
+
+
+__all__ = [
+    'Preprocess',
+    'AnnotationJitter', 'NormalizeAnnotations',
+    'Compose',
+    'Crop',
+    'Encoders',
+    'HFlip',
+    'Blur', 'ImageTransform', 'JpegCompression',
+    'MinSize',
+    'MultiScale',
+    'CenterPad', 'CenterPadTight', 'SquarePad',
+    'DeterministicEqualChoice', 'RandomApply', 'RandomChoice',
+    'RotateBy90', 'RotateUniform',
+    'RescaleAbsolute', 'RescaleRelative', 'ScaleMix',
+    'ToAnnotations', 'ToCrowdAnnotations', 'ToDetAnnotations', 'ToKpAnnotations',
+    'UnclippedArea', 'UnclippedSides',
+]
