@@ -9,6 +9,11 @@ import torch
 
 from .preprocess import Preprocess
 
+try:
+    import cv2
+except ImportError:
+    cv2 = None
+
 LOG = logging.getLogger(__name__)
 
 
@@ -26,7 +31,22 @@ def _scale(image, anns, meta, target_w, target_h, resample, *, fast=False):
     assert resample in (0, 2, 3)
 
     # scale image
-    if fast:
+    if fast and cv2 is not None:
+        LOG.debug('using OpenCV for fast rescale')
+        if resample == 0:
+            cv_interpoltation = cv2.INTER_NEAREST
+        elif resample == 2:
+            cv_interpoltation = cv2.INTER_LINEAR
+        elif resample == 3:
+            cv_interpoltation = cv2.INTER_CUBIC
+        else:
+            raise NotImplementedError('resample of {} not implemented for OpenCV'.format(resample))
+        im_np = np.asarray(image)
+        im_np = cv2.resize(im_np, (target_w, target_h), interpolation=cv_interpoltation)
+        image = PIL.Image.fromarray(im_np)
+    elif fast:
+        LOG.debug('Requested fast resizing without OpenCV. Using Pillow. '
+                  'Install OpenCV for even faster image resizing.')
         image = image.resize((target_w, target_h), resample)
     else:
         order = resample
@@ -67,24 +87,28 @@ def _scale(image, anns, meta, target_w, target_h, resample, *, fast=False):
 
 
 class RescaleRelative(Preprocess):
+    """Rescale relative to input image."""
+
     def __init__(self, scale_range=(0.5, 1.0), *,
                  resample=PIL.Image.BILINEAR,
                  absolute_reference=None,
                  fast=False,
-                 power_law=False):
+                 power_law=False,
+                 stretch_range=None):
         self.scale_range = scale_range
         self.resample = resample
         self.absolute_reference = absolute_reference
         self.fast = fast
         self.power_law = power_law
+        self.stretch_range = stretch_range
 
     def __call__(self, image, anns, meta):
         if isinstance(self.scale_range, tuple):
             if self.power_law:
                 rnd_range = np.log2(self.scale_range[0]), np.log2(self.scale_range[1])
                 log2_scale_factor = (
-                    rnd_range[0] +
-                    torch.rand(1).item() * (rnd_range[1] - rnd_range[0])
+                    rnd_range[0]
+                    + torch.rand(1).item() * (rnd_range[1] - rnd_range[0])
                 )
 
                 scale_factor = 2 ** log2_scale_factor
@@ -92,8 +116,8 @@ class RescaleRelative(Preprocess):
                           rnd_range, log2_scale_factor, scale_factor)
             else:
                 scale_factor = (
-                    self.scale_range[0] +
-                    torch.rand(1).item() * (self.scale_range[1] - self.scale_range[0])
+                    self.scale_range[0]
+                    + torch.rand(1).item() * (self.scale_range[1] - self.scale_range[0])
                 )
         else:
             scale_factor = self.scale_range
@@ -106,11 +130,21 @@ class RescaleRelative(Preprocess):
             else:
                 w *= self.absolute_reference / h
                 h = self.absolute_reference
-        target_w, target_h = int(w * scale_factor), int(h * scale_factor)
+
+        stretch_factor = 1.0
+        if self.stretch_range is not None:
+            stretch_factor = (
+                self.stretch_range[0]
+                + torch.rand(1).item() * (self.stretch_range[1] - self.stretch_range[0])
+            )
+
+        target_w, target_h = int(w * scale_factor * stretch_factor), int(h * scale_factor)
         return _scale(image, anns, meta, target_w, target_h, self.resample, fast=self.fast)
 
 
 class RescaleAbsolute(Preprocess):
+    """Rescale to a given size."""
+
     def __init__(self, long_edge, *, fast=False, resample=PIL.Image.BILINEAR):
         self.long_edge = long_edge
         self.fast = fast
@@ -147,8 +181,8 @@ class ScaleMix(Preprocess):
     def __call__(self, image, anns, meta):
         scales = np.array([
             np.sqrt(ann['bbox'][2] * ann['bbox'][3])
-            for ann in anns if (not getattr(ann, 'iscrowd', False) and
-                                np.any(ann['keypoints'][:, 2] > 0.0))
+            for ann in anns if (not getattr(ann, 'iscrowd', False)
+                                and np.any(ann['keypoints'][:, 2] > 0.0))
         ])
         LOG.debug('scale threshold = %f, scales = %s', self.scale_threshold, scales)
         if not scales.shape[0]:
